/*
	Little Smalltalk, version 3
	Written by Tim Budd, January 1989

	rmboard interface routines base on tty interface routines
	this is used by those systems that have a iXs Research RM-board
        interface.
*/

# include <stdio.h>
# include "env.h"
# include "memory.h"
# include "names.h"
# include "sbus.h"
# include "rmeye.h"
# include "gpio.h"
#include <stdlib.h>

extern boolean parseok;

/* report a fatal system error */
noreturn sysError(s1, s2)
char *s1, *s2;
{
	ignore fprintf(stderr,"%s\n%s\n", s1, s2);
	ignore abort();
}

/* report a nonfatal system error */
noreturn sysWarn(s1, s2)
char *s1, *s2;
{
	ignore fprintf(stderr,"%s\n%s\n", s1, s2);
}

compilWarn(selector, str1, str2)
char *selector, *str1, *str2;
{
	ignore fprintf(stderr,"compiler warning: Method %s : %s %s\n", 
		selector, str1, str2);
}

compilError(selector, str1, str2)
char *selector, *str1, *str2;
{
	ignore fprintf(stderr,"compiler error: Method %s : %s %s\n", 
		selector, str1, str2);
	parseok = false;
}

noreturn dspMethod(cp, mp)
char *cp, *mp;
{
	/*ignore fprintf(stderr,"%s %s\n", cp, mp);*/
}

givepause()
{
	ignore fprintf(stderr,"[givepause]\n");
}

#define safe16384(x) ((x)>16383?16383:((x)<-16383?-16383:(x)))

object sysPrimitive(int number, object *arguments)
{
  object returnedObject = nilobj;
  int num = 0;
  int value = 0;
  int v2 = 0, v3 = 0;

  switch(number) {
  case 160: /* sbus_init */
    returnedObject = newInteger(sbus_init());
    break;
  case 161: /* sbus_reset */
    returnedObject = newInteger(sbus_reset());
    break;
  case 162: /* sbus_setpwm */
    returnedObject = newInteger(sbus_setpwm());
    break;
  case 163: /* sbus_getwidth */
    returnedObject = newInteger(sbus_getwidth());
    break;
  case 164: /* sbus_setwidth */
    returnedObject = newInteger(sbus_setwidth());
    break;
  case 165: /* sbus_setserv */
    num = intValue(arguments[0]);
    value = intValue(arguments[1]);
    sbus_setserv( num, PWM_DEG * value );
    break;
  case 166: /* sbus_getserv */
    num = intValue(arguments[0]);
    value = sbus_getserv(num);
    value /= PWM_DEG;
    returnedObject = newInteger(value);
    break;
  case 167: /* sbus_addserv */
    num = intValue(arguments[0]);
    value = intValue(arguments[1]);
    v2 = sbus_addserv(num, PWM_DEG * value);
    v2 /= PWM_DEG;
    returnedObject = newInteger(v2);
    break;
  case 168: /* sbus_getad */
    num = intValue(arguments[0]);
    value = sbus_getad(num);
    returnedObject = newInteger(value);
    break;
  case 169: /* sbus_initnext */
    sbus_initnext();
    break;
  case 170: /* sbus_setnext */
    num = intValue(arguments[0]);
    value = intValue(arguments[1]);
    sbus_setnext( num, PWM_DEG * value );
    break;
  case 171: /* sbus_getnext */
    num = intValue(arguments[0]);
    value = sbus_getnext(num);
    value /= PWM_DEG;
    returnedObject = newInteger(value);
    break;
  case 172: /* sbus_stepnext */
    num = intValue(arguments[0]);
    value = sbus_stepnext(num);
    returnedObject = newInteger(value);
    break;
  case 173: /* sbus_movesync */
    num = intValue(arguments[0]);
    sbus_movesync(num);
    break;
  case 174: /* sbus_usetrim */
    num = intValue(arguments[0]);
    sbus_usetrim(num);
    break;
  case 175: /* sbus_printnext */
    sbus_printnext();
    break;

  case 180: /* rmeye_init */
    returnedObject = newInteger(rmeye_init());
    break;
  case 181: /* rmeye_getpoint */
    num = intValue(arguments[0]);
    num = rmeye_getpoint( num, &value, &v2, &v3 );
    if ( num < 0 ) {
      break;
    }
    v3 = safe16384(v3);
    returnedObject = newArray(3);
    basicAtPut(returnedObject, 1, newInteger(value));
    basicAtPut(returnedObject, 2, newInteger(v2));
    basicAtPut(returnedObject, 3, newInteger(v3));
    break;
  case 182: /* sleep */
    num = intValue(arguments[0]);
    sleep( num );
    break;

  case 190: /* gpio_getsw */
    num = intValue(arguments[0]);
    returnedObject = newInteger(gpio_getsw(num));
    break;
  case 191: /* gpio_setled */
    num = intValue(arguments[0]);
    v2 = intValue(arguments[1]);
    gpio_setled(num, v2);
    break;

  default:
    sysWarn("unknown primitive","sysPrimitive");
  }
  return returnedObject;
}
